@@ -6,6 +6,23 @@
 #include "util/windows/unicode.h"
 #include "localip.h"
 
+#define FF_LOCALIP_NIFLAG(name) { IP_ADAPTER_##name, #name }
+
+static const FFLocalIpNIFlag niFlagOptions[] = {
+    FF_LOCALIP_NIFLAG(DDNS_ENABLED),
+    FF_LOCALIP_NIFLAG(REGISTER_ADAPTER_SUFFIX),
+    FF_LOCALIP_NIFLAG(DHCP_ENABLED),
+    FF_LOCALIP_NIFLAG(RECEIVE_ONLY),
+    FF_LOCALIP_NIFLAG(NO_MULTICAST),
+    FF_LOCALIP_NIFLAG(IPV6_OTHER_STATEFUL_CONFIG),
+    FF_LOCALIP_NIFLAG(NETBIOS_OVER_TCPIP_ENABLED),
+    FF_LOCALIP_NIFLAG(IPV4_ENABLED),
+    FF_LOCALIP_NIFLAG(IPV6_ENABLED),
+    FF_LOCALIP_NIFLAG(IPV6_MANAGE_ADDRESS_CONFIG),
+    // sentinel
+    {},
+};
+
 static void addNewIp(FFlist* list, const char* name, const char* addr, int type, bool newIp, bool defaultRoute)
 {
     FFLocalIpResult* ip = NULL;
@@ -17,6 +34,7 @@ static void addNewIp(FFlist* list, const char* name, const char* addr, int type,
         ffStrbufInit(&ip->ipv4);
         ffStrbufInit(&ip->ipv6);
         ffStrbufInit(&ip->mac);
+        ffStrbufInit(&ip->flags);
         ip->defaultRoute = defaultRoute;
         ip->speed = -1;
         ip->mtu = -1;
@@ -74,11 +92,14 @@ const char* ffDetectLocalIps(const FFLocalIpOptions* options, FFlist* results)
             return "GetAdaptersAddresses() failed";
     }
 
-    uint32_t defaultRouteIfIndex = ffNetifGetDefaultRouteIfIndex();
+    uint32_t defaultRouteIfIndex = ffNetifGetDefaultRouteV4()->ifIndex;
 
     // Iterate through all of the adapters
     for (IP_ADAPTER_ADDRESSES* adapter = adapter_addresses; adapter; adapter = adapter->Next)
     {
+        if (adapter->OperStatus != IfOperStatusUp)
+            continue;
+
         bool isDefaultRoute = adapter->IfIndex == defaultRouteIfIndex;
         if ((options->showType & FF_LOCALIP_TYPE_DEFAULT_ROUTE_ONLY_BIT) && !isDefaultRoute)
             continue;
@@ -90,7 +111,7 @@ const char* ffDetectLocalIps(const FFLocalIpOptions* options, FFlist* results)
         bool newIp = true;
 
         char name[128];
-        WideCharToMultiByte(CP_UTF8, 0, adapter->FriendlyName, -1, name, sizeof(name), NULL, NULL);
+        WideCharToMultiByte(CP_UTF8, 0, adapter->FriendlyName, -1, name, ARRAY_SIZE(name), NULL, NULL);
         if (options->namePrefix.length && strncmp(name, options->namePrefix.chars, options->namePrefix.length) != 0)
             continue;
 
@@ -98,7 +119,7 @@ const char* ffDetectLocalIps(const FFLocalIpOptions* options, FFlist* results)
         {
             char addressBuffer[32];
             uint8_t* ptr = adapter->PhysicalAddress;
-            snprintf(addressBuffer, sizeof(addressBuffer), "%02x:%02x:%02x:%02x:%02x:%02x",
+            snprintf(addressBuffer, ARRAY_SIZE(addressBuffer), "%02x:%02x:%02x:%02x:%02x:%02x",
                         ptr[0], ptr[1], ptr[2], ptr[3], ptr[4], ptr[5]);
             addNewIp(results, name, addressBuffer, -1, newIp, isDefaultRoute);
             newIp = false;
@@ -108,18 +129,21 @@ const char* ffDetectLocalIps(const FFLocalIpOptions* options, FFlist* results)
 
         for (IP_ADAPTER_UNICAST_ADDRESS* ifa = adapter->FirstUnicastAddress; ifa; ifa = ifa->Next)
         {
+            if (!(options->showType & FF_LOCALIP_TYPE_ALL_IPS_BIT) && ifa->DadState != IpDadStatePreferred)
+                continue;
+
             if (ifa->Address.lpSockaddr->sa_family == AF_INET)
             {
                 if (!(typesToAdd & (FF_LOCALIP_TYPE_IPV4_BIT | FF_LOCALIP_TYPE_ALL_IPS_BIT))) continue;
 
                 SOCKADDR_IN* ipv4 = (SOCKADDR_IN*) ifa->Address.lpSockaddr;
-                char addressBuffer[INET_ADDRSTRLEN + 4];
+                char addressBuffer[INET_ADDRSTRLEN + 6];
                 inet_ntop(AF_INET, &ipv4->sin_addr, addressBuffer, INET_ADDRSTRLEN);
 
                 if ((options->showType & FF_LOCALIP_TYPE_PREFIX_LEN_BIT) && ifa->OnLinkPrefixLength)
                 {
                     size_t len = strlen(addressBuffer);
-                    snprintf(addressBuffer + len, 4, "/%u", (unsigned) ifa->OnLinkPrefixLength);
+                    snprintf(addressBuffer + len, 6, "/%u", (unsigned) ifa->OnLinkPrefixLength);
                 }
 
                 addNewIp(results, name, addressBuffer, AF_INET, newIp, isDefaultRoute);
@@ -133,13 +157,13 @@ const char* ffDetectLocalIps(const FFLocalIpOptions* options, FFlist* results)
                 if (!(typesToAdd & (FF_LOCALIP_TYPE_IPV6_BIT | FF_LOCALIP_TYPE_ALL_IPS_BIT))) continue;
 
                 SOCKADDR_IN6* ipv6 = (SOCKADDR_IN6*) ifa->Address.lpSockaddr;
-                char addressBuffer[INET6_ADDRSTRLEN + 4];
+                char addressBuffer[INET6_ADDRSTRLEN + 6];
                 inet_ntop(AF_INET6, &ipv6->sin6_addr, addressBuffer, INET6_ADDRSTRLEN);
 
                 if ((options->showType & FF_LOCALIP_TYPE_PREFIX_LEN_BIT) && ifa->OnLinkPrefixLength)
                 {
                     size_t len = strlen(addressBuffer);
-                    snprintf(addressBuffer + len, 4, "/%u", (unsigned) ifa->OnLinkPrefixLength);
+                    snprintf(addressBuffer + len, 6, "/%u", (unsigned) ifa->OnLinkPrefixLength);
                 }
 
                 addNewIp(results, name, addressBuffer, AF_INET6, newIp, isDefaultRoute);
@@ -153,8 +177,12 @@ const char* ffDetectLocalIps(const FFLocalIpOptions* options, FFlist* results)
         if (!newIp)
         {
             FFLocalIpResult* result = FF_LIST_GET(FFLocalIpResult, *results, results->length - 1);
-            result->speed = (int32_t) (adapter->ReceiveLinkSpeed / 1000);
-            result->mtu = (int32_t) adapter->Mtu;
+            if (options->showType & FF_LOCALIP_TYPE_SPEED_BIT)
+                result->speed = (int32_t) (adapter->ReceiveLinkSpeed / 1000000);
+            if (options->showType & FF_LOCALIP_TYPE_MTU_BIT)
+                result->mtu = (int32_t) adapter->Mtu;
+            if (options->showType & FF_LOCALIP_TYPE_FLAGS_BIT)
+                ffLocalIpFillNIFlags(&result->flags, adapter->Flags, niFlagOptions);
         }
     }
 
