@@ -53,11 +53,12 @@ static pid_t getShellInfo(FFShellResult* result, pid_t pid)
                 ffStrbufContainS(&result->processName, "hyfetch")           || //when hyfetch uses fastfetch as backend
                 ffStrbufEqualS(&result->processName, "clifm")               || //https://github.com/leo-arch/clifm/issues/289
                 ffStrbufEqualS(&result->processName, "valgrind")            ||
-                ffStrbufEqualS(&result->processName, "fastfetch")           || //994
+                ffStrbufEqualS(&result->processName, "fastfetch")           || //#994
                 ffStrbufEqualS(&result->processName, "flashfetch")          ||
+                ffStrbufEqualS(&result->processName, "proot")               ||
+                ffStrbufEqualS(&result->processName, "script")              ||
                 ffStrbufContainS(&result->processName, "debug")             ||
                 ffStrbufContainS(&result->processName, "command-not-")      ||
-                ffStrbufEqualS(&result->processName, "proot")              ||
                 ffStrbufEndsWithS(&result->processName, ".sh")
             )
             {
@@ -107,6 +108,9 @@ static pid_t getTerminalInfo(FFTerminalResult* result, pid_t pid)
             ffStrbufEqualS(&result->processName, "clifm")      || // https://github.com/leo-arch/clifm/issues/289
             ffStrbufEqualS(&result->processName, "chezmoi")    || // #762
             ffStrbufEqualS(&result->processName, "proot")      ||
+            ffStrbufEqualS(&result->processName, "script")     ||
+            ffStrbufEqualS(&result->processName, "init")       ||
+            ffStrbufEqualS(&result->processName, "systemd")    ||
             #ifdef __linux__
             ffStrbufStartsWithS(&result->processName, "flatpak-") || // #707
             #endif
@@ -170,7 +174,6 @@ static void getTerminalFromEnv(FFTerminalResult* result)
 
             #ifdef __APPLE__
             !ffStrbufEqualS(&result->processName, "launchd") &&
-            !ffStrbufEqualS(&result->processName, "stable") && //for WarpTerminal
             #else
             !ffStrbufEqualS(&result->processName, "systemd") &&
             !ffStrbufEqualS(&result->processName, "init") &&
@@ -351,14 +354,24 @@ static void setTerminalInfoDetails(FFTerminalResult* result)
         ffStrbufInitStatic(&result->prettyName, "iTerm");
     else if(ffStrbufEndsWithS(&result->exePath, "Terminal.app/Contents/MacOS/Terminal"))
     {
-        ffStrbufSetStatic(&result->processName, "Apple_Terminal"); // for terminal font detection
+        ffStrbufSetStatic(&result->processName, "Apple_Terminal"); // $TERM_PROGRAM, for terminal font detection
         ffStrbufInitStatic(&result->prettyName, "Apple Terminal");
     }
     else if(ffStrbufEqualS(&result->processName, "Apple_Terminal"))
         ffStrbufInitStatic(&result->prettyName, "Apple Terminal");
+    else if(ffStrbufEndsWithS(&result->exePath, "Warp.app/Contents/MacOS/stable"))
+    {
+        ffStrbufSetStatic(&result->processName, "WarpTerminal"); // $TERM_PROGRAM, for terminal font detection
+        ffStrbufInitStatic(&result->prettyName, "Warp");
+    }
     else if(ffStrbufEqualS(&result->processName, "WarpTerminal"))
         ffStrbufInitStatic(&result->prettyName, "Warp");
 
+    #elif defined(__HAIKU__)
+
+    else if(ffStrbufEqualS(&result->processName, "Terminal"))
+        ffStrbufInitStatic(&result->prettyName, "Haiku Terminal");
+
     #endif
 
     else if(strncmp(result->exeName, result->processName.chars, result->processName.length) == 0) // if exeName starts with processName, print it. Otherwise print processName
