@@ -1,9 +1,11 @@
 #pragma once
 
-#include "fastfetch.h"
+#include "option.h"
 
 #define FF_BOOTMGR_MODULE_NAME "Bootmgr"
 
 void ffPrintBootmgr(FFBootmgrOptions* options);
 void ffInitBootmgrOptions(FFBootmgrOptions* options);
 void ffDestroyBootmgrOptions(FFBootmgrOptions* options);
+
+extern FFModuleBaseInfo ffBootmgrModuleInfo;
