@@ -1,9 +1,11 @@
 #pragma once
 
-#include "fastfetch.h"
+#include "option.h"
 
 #define FF_VULKAN_MODULE_NAME "Vulkan"
 
 void ffPrintVulkan(FFVulkanOptions* options);
 void ffInitVulkanOptions(FFVulkanOptions* options);
 void ffDestroyVulkanOptions(FFVulkanOptions* options);
+
+extern FFModuleBaseInfo ffVulkanModuleInfo;
